// Determines which partition each Universe element should be in, from a given HC from the converted instance.
//
// The input should be a .hcp.tou file, such as the following:
//
// NAME : filename.hcp.tou
// COMMENT : HC for filename.hcp
// TYPE : TOUR
// DIMENSION : 1200
// TOUR_SECTION
// 1
// 2
// 3
// ...
// 1200
// -1
// EOF
//
//
// Written by Michael Haythorpe

import java.io.*;
import java.util.StringTokenizer;

public class SSP2HCPSol
{
   public static void main(String [] args)
   {
      if(args.length != 1)
      {
         System.out.println("ERROR: Input must be of the following form: SSP2HCPSol <output.hcp.tou>");
         return;
      }
   
      String outputFilename = args[0];
      BufferedReader br;
      
           
      try
      {
         br = new BufferedReader(new FileReader(outputFilename));
      }
      catch(Exception e)
      {
         System.out.println("ERROR: Cannot open file " + outputFilename);
         return;
      }
         
      String line = "";
   		
      try
      {
         line = br.readLine();
         line = br.readLine();
         line = br.readLine();
         line = br.readLine();
         StringTokenizer tokens = new StringTokenizer(line);
         tokens.nextToken();
         tokens.nextToken();
         int nodes = Integer.parseInt(tokens.nextToken());
         line = br.readLine();
         line = br.readLine();
         int []HC = new int[nodes];
         for(int i=0; i<nodes; i++)
         {
            HC[i] = Integer.parseInt(line);
            line = br.readLine();   
         }
         
         if(HC[1] != 2)
         {
            if(HC[nodes-1] != 2)
            {
               System.out.println("Not a valid HC!");
               return;
            }
            int HC2[] = new int[nodes];
            HC2[0] = 1;
            for(int i=1; i<nodes; i++)
               HC2[i] = HC[nodes-i];
            HC = HC2;
         }
         
         int count = 0;
         int element = 1;
         while(true)
         {
            while(HC[count] != 12*element - 9)
               count++;
         
            if(HC[count+3] == 12*element - 6 && HC[count+6] == 12*element - 3 && HC[count+9] != 12*element)
               System.out.println("Element " + element + " in set U1");
            else if(HC[count+3] == 12*element - 3 && HC[count+6] == 12*element - 6)
               System.out.println("Element " + element + " in set U2");
            else
            {
               br.close();
               return;
            }
            
            element++;
         }
         
         
      }
      catch(Exception e)
      {
         System.out.println(e);
      }
   }
}